@@ -18,6 +18,8 @@ from testflows._core.cli.arg.handlers.transform.nice import Handler as nice_hand
 from testflows._core.cli.arg.handlers.transform.short import Handler as short_handler
 from testflows._core.cli.arg.handlers.transform.dots import Handler as dots_handler
 from testflows._core.cli.arg.handlers.transform.index import Handler as index_handler
+from testflows._core.cli.arg.handlers.transform.compact import Handler as compact_handler
+from testflows._core.cli.arg.handlers.transform.slick import Handler as slick_handler
 
 class Handler(HandlerBase):
     @classmethod
@@ -31,5 +33,7 @@ class Handler(HandlerBase):
         transform_commands.required = True
         nice_handler.add_command(transform_commands)
         short_handler.add_command(transform_commands)
+        slick_handler.add_command(transform_commands)
         dots_handler.add_command(transform_commands)
-        index_handler.add_command(transform_commands)
\ No newline at end of file
+        index_handler.add_command(transform_commands)
+        compact_handler.add_command(transform_commands)
