@@ -12,7 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 from argparse import FileType
+from argparse import ArgumentTypeError
+
+from collections import namedtuple
+
+KeyValue = namedtuple("KeyValue", "key value")
 
 def file(*args, **kwargs):
     """File type."""
     return FileType(*args, **kwargs)
+
+def key_value(s, sep='='):
+    """Parse a key, value pair using a seperator (default: '=').
+    """
+    if sep not in s:
+        raise ArgumentTypeError(f"invalid format of key{sep}value")
+    key, value= s.split(sep, 1)
+    return KeyValue(key.strip(), value.strip())
