@@ -205,3 +205,10 @@ def enter_context(cm, test=None):
     if not test.caller_test or test.caller_test.subtype != TestSubType.Background:
         raise TypeError("not inside a background test")
     return test.caller_test.stack.enter_context(cm)
+
+def getsattr(obj, name, *default):
+    """Get attribute or set it to the default value.
+    """
+    value = getattr(obj, name, *default)
+    setattr(obj, name, value)
+    return value
\ No newline at end of file
