@@ -28,7 +28,7 @@ from .exceptions import DummyTestException, ArgumentError, ResultException, Repe
 from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
-from .objects import get, Null, OK, Fail, Skip, Error, Argument, ExamplesTable
+from .objects import get, Null, OK, Fail, Skip, Error, Argument, Attribute, ExamplesTable
 from .constants import name_sep, id_sep
 from .io import TestIO, LogWriter
 from .name import join, depth, match, absname
@@ -36,6 +36,7 @@ from .funcs import current, top, previous, main, skip, ok, fail, error, exceptio
 from .init import init
 from .cli.arg.parser import ArgumentParser
 from .cli.arg.exit import ExitWithError, ExitException
+from .cli.arg.type import key_value as key_value_type
 from .cli.text import danger, warning
 from .exceptions import exception as get_exception
 from .filters import the
@@ -98,6 +99,72 @@ class DummyTest(object):
         if isinstance(exception_value, DummyTestException):
             return True
 
+class TestContext(object):
+    """Test context.
+    """
+    def __init__(self, parent, state=None):
+        self._parent = parent
+        self._state = get(state, {})
+        self._cleanups = []
+
+    def cleanup(self, func, *args, **kwargs):
+        def func_wrapper():
+            func(*args, **kwargs)
+        self._cleanups.append(func_wrapper)
+
+    def _cleanup(self):
+        exc_type, exc_value, exc_traceback = None, None, None
+        for func in reversed(self._cleanups):
+            try:
+                func()
+            except StopIteration:
+                pass
+            except (Exception, KeyboardInterrupt) as e:
+                if not exc_value:
+                    exc_type, exc_value, exc_traceback = sys.exc_info()
+        return exc_type, exc_value, exc_traceback
+
+    def __getattr__(self, name):
+        try:
+            if name.startswith('_'):
+                return self.__dict__[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+        curr = self
+        while True:
+            try:
+                return curr._state[name]
+            except KeyError:
+                if curr._parent:
+                    curr = curr._parent
+                    continue
+                raise AttributeError(name) from None
+
+    def __setattr__(self, name, value):
+        if name.startswith('_'):
+            self.__dict__[name] = value
+        else:
+            self._state[name] = value
+
+    def __delattr__(self, name):
+        try:
+            del self._state[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+    def __contains__(self, name):
+        if name.startswith('_'):
+            return name in self.__dict__
+
+        curr = self
+        while True:
+            if name in curr._state:
+                return True
+            if curr._parent:
+                curr = curr._parent
+            else:
+                return False
 
 class TestBase(object):
     """Base class for all the tests.
@@ -118,6 +185,8 @@ class TestBase(object):
     examples = None
     name = None
     description = None
+    node = None
+    map = None
     flags = Flags()
     name_sep = "."
     type = TestType.Test
@@ -148,6 +217,12 @@ class TestBase(object):
                 description=((cls.description or "") + definitions_description),
                 description_prog="Test - Framework"
             )
+        parser.add_argument("--name", dest="_name", metavar="name",
+            help="test run name", type=str, required=False)
+        parser.add_argument("--tag", dest="_tags", metavar="value", nargs="+",
+            help="test run tags", type=str, required=False)
+        parser.add_argument("--attr", dest="_attrs", metavar="name=value", nargs="+",
+            help="test run attributes", type=key_value_type, required=False)
         parser.add_argument("--only", dest="_only", metavar="pattern", nargs="+",
             help="run only selected tests", type=str, required=False)
         parser.add_argument("--skip", dest="_skip", metavar="pattern", nargs="+",
@@ -166,8 +241,8 @@ class TestBase(object):
             help="disable terminal color highlighting", default=False)
         parser.add_argument("--id", metavar="id", dest="_id", type=str, help="custom test id")
         parser.add_argument("-o", "--output", dest="_output", metavar="format", type=str,
-            choices=["nice", "quiet", "short", "dots", "raw", "silent"], default="nice",
-            help="""stdout output format, choices are: ['nice','short','dots','quiet','raw','silent'],
+            choices=["slick", "nice", "quiet", "short", "dots", "raw", "silent"], default="nice",
+            help="""stdout output format, choices are: ['slick','nice','short','dots','quiet','raw','silent'],
                 default: 'nice'""")
         parser.add_argument("-l", "--log", dest="_log", metavar="file", type=str,
             help="path to the log file where test output will be stored, default: uses temporary log file")
@@ -175,7 +250,8 @@ class TestBase(object):
             help="show skipped tests, default: False", default=False)
         return parser
 
-    def parse_cli_args(self, xflags=None, only=None, skip=None, start=None, end=None):
+    def parse_cli_args(self, xflags=None, only=None, skip=None, start=None, end=None,
+            name=None, tags=None, attributes=None):
         """Parse command line arguments.
 
         :return: parsed known arguments
@@ -185,6 +261,12 @@ class TestBase(object):
             args, unknown = parser.parse_known_args()
             args = vars(args)
 
+            if args.get("_name"):
+                name = args.pop("_name")
+
+            if name is None:
+                raise TypeError("name must be specified")
+
             if args.get("_debug"):
                 settings.debug = True
                 args.pop("_debug")
@@ -221,16 +303,16 @@ class TestBase(object):
                 if not xflags:
                     xflags = globals()["xflags"]()
                 for pattern in args.get("_pause_before"):
-                    pattern = absname(pattern, self.name)
+                    pattern = absname(pattern, name)
                     xflags[pattern] = xflags.get(pattern, [0, 0])
                     xflags[pattern][0] |= PAUSE_BEFORE
                 args.pop("_pause_before")
 
             if args.get("_pause_after"):
                 if not xflags:
-                    xflags = globals()["xflags"](xflags)
+                    xflags = globals()["xflags"]()
                 for pattern in args.get("_pause_after"):
-                    pattern = absname(pattern, self.name)
+                    pattern = absname(pattern, name)
                     xflags[pattern] = xflags.get(pattern, [0, 0])
                     xflags[pattern][0] |= PAUSE_AFTER
                 args.pop("_pause_after")
@@ -238,23 +320,29 @@ class TestBase(object):
             if args.get("_only"):
                 only = [] # clear whatever was passed
                 for pattern in args.get("_only"):
-                    only.append(the(pattern).at(self.name))
+                    only.append(the(pattern).at(name))
                 args.pop("_only")
 
             if args.get("_skip"):
                 skip = [] # clear whatever was passed
                 for pattern in args.get("_skip"):
-                    only.append(the(pattern).at(self.name))
+                    only.append(the(pattern).at(name))
                 args.pop("_skip")
 
             if args.get("_start"):
-                start = the(args.get("_start")[0]).at(self.name)
+                start = the(args.get("_start")[0]).at(name)
                 args.pop("_start")
 
             if args.get("_end"):
-                end = the(args.get("_end")[0]).at(self.name)
+                end = the(args.get("_end")[0]).at(name)
                 args.pop("_end")
 
+            if args.get("_tags"):
+                tags = [value for value in args.pop("_tags")]
+
+            if args.get("_attrs"):
+                attributes = [Attribute(item.key, item.value) for item in args.pop("_attrs")]
+
         except (ExitException, KeyboardInterrupt, Exception) as exc:
             #if settings.debug:
             sys.stderr.write(warning(get_exception(), eol='\n'))
@@ -264,19 +352,17 @@ class TestBase(object):
             else:
                 sys.exit(1)
 
-        return args, xflags, only, skip, start, end
+        return args, xflags, only, skip, start, end, name, tags, attributes
 
     def __init__(self, name=None, flags=None, cflags=None, type=None, subtype=None,
                  uid=None, tags=None, attributes=None, requirements=None,
                  users=None, tickets=None, examples=None, description=None, parent=None,
                  xfails=None, xflags=None, only=None, skip=None,
-                 start=None, end=None, args=None, id=None, _frame=None, _run=True):
+                 start=None, end=None, args=None, id=None, node=None, map=None, context=None,
+                 _frame=None, _run=True):
 
         self.lock = threading.Lock()
-        self.name = name
         self._run = _run
-        if self.name is None:
-            raise TypeError("name must be specified")
 
         cli_args = {}
         if current() is None:
@@ -287,12 +373,19 @@ class TestBase(object):
             self._init= False
             frame = get(_frame, inspect.currentframe().f_back.f_back.f_back)
             if main(frame):
-                cli_args, xflags, only, skip, start, end = self.parse_cli_args(xflags, only, skip, start, end)
+                cli_args, xflags, only, skip, start, end, name, tags, attributes = self.parse_cli_args(
+                    xflags, only, skip, start, end, name, tags, attributes)
 
+        self.name = name
+        if self.name is None:
+            raise TypeError("name must be specified")
         self.child_count = 0
         self.start_time = time.time()
         self.parent = parent
         self.id = get(id, [settings.test_id])
+        self.node = get(node, self.node)
+        self.map = get(map, self.map)
+        self.context = get(context, TestContext(current().context if current() else None))
         self.tags = tags
         self.requirements = get(requirements, self.requirements)
         self.attributes = get(attributes, self.attributes)
@@ -326,6 +419,8 @@ class TestBase(object):
         :param parent: parent name
         """
         name = name % {"name": cls.name} if name is not None else cls.name
+        if name is None:
+            raise TypeError("name must be specified")
         # '/' is not allowed just like in Unix file names
         # so convert any '/' to U+2215 division slash
         name = name.replace(name_sep, "\u2215")
@@ -385,29 +480,39 @@ class TestBase(object):
         if top() is self and not self._init:
             return False
 
-        current(self.caller_test)
-        previous(self)
+        def process_exception(exception_type, exception_value, exception_traceback):
+            if isinstance(exception_value, ResultException):
+                self.result = exception_value.result
+            elif isinstance(exception_value, AssertionError):
+                exception(test=self)
+                self.result = Fail(self.name, str(exception_value))
+            else:
+                exception(test=self)
+                self.result = Error(self.name,
+                                    "unexpected %s: %s" % (exception_type.__name__, str(exception_value)))
+                if isinstance(exception_value, KeyboardInterrupt):
+                    raise ResultException(self.result)
 
         try:
             if exception_value:
-                if isinstance(exception_value, ResultException):
-                    self.result = exception_value.result
-                elif isinstance(exception_value, AssertionError):
-                    exception(test=self)
-                    self.result = Fail(self.name, str(exception_value).split('\n', 1)[0])
-                else:
-                    exception(test=self)
-                    self.result = Error(self.name,
-                        "unexpected %s: %s" % (exception_type.__name__, str(exception_value).split('\n', 1)[0]))
-                    if isinstance(exception_value, KeyboardInterrupt):
-                        raise ResultException(self.result)
+                process_exception(exception_type, exception_value, exception_traceback)
             else:
                 if isinstance(self.result, Null):
                     self.result = OK(self.name)
         finally:
-            self._apply_xfails()
-            self.io.output.result(self.result)
-            self.io.close()
+            try:
+                if self.type >= TestType.Test:
+                    if self.context._cleanups:
+                        with Finally("I clean up"):
+                            cleanup_exc_type, cleanup_exc_value, cleanup_exc_traceback = self.context._cleanup()
+                        if not exception_value and cleanup_exc_value:
+                            process_exception(cleanup_exc_type, cleanup_exc_value, cleanup_exc_traceback)
+            finally:
+                current(self.caller_test)
+                previous(self)
+                self._apply_xfails()
+                self.io.output.result(self.result)
+                self.io.close()
 
             if self.flags & PAUSE_AFTER:
                 pause()
@@ -748,13 +853,22 @@ class Scenario(Test):
         kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
         return super(Scenario, self).__init__(name, **kwargs)
 
-
 class _background(Test):
     def __init__(self, name, **kwargs):
         kwargs["subtype"] = TestSubType.Background
         kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back.f_back)
         return super(_background, self).__init__(name, **kwargs)
 
+class Steps(ExitStack):
+    def __init__(self, *args, **kwargs):
+        self.values = []
+        super(Steps, self).__init__(*args, **kwargs)
+
+    def __call__(self, ctx):
+        step = self.enter_context(ctx)
+        self.values.append(step)
+        return step
+
 @contextmanager
 def Background(name, **kwargs):
     with _background(name, **kwargs) as bg:
@@ -762,47 +876,68 @@ def Background(name, **kwargs):
             bg.stack = stack
             yield bg
 
-class Given(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.Given
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(Given, self).__init__(name, **kwargs)
+def Given(name, **kwargs):
+    kwargs["subtype"] = TestSubType.Given
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
-class When(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.When
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(When, self).__init__(name, **kwargs)
+def When(step, **kwargs):
+    kwargs["subtype"] = TestSubType.When
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(step, TestStep):
+        return step(**kwargs)
+    else:
+        return Step(step, **kwargs)
 
-class Then(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.Then
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(Then, self).__init__(name, **kwargs)
+def Then(name, **kwargs):
+    kwargs["subtype"] = TestSubType.Then
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
-class And(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.And
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(And, self).__init__(name, **kwargs)
+def And(name, **kwargs):
+    kwargs["subtype"] = TestSubType.And
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
-class But(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.But
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(But, self).__init__(name, **kwargs)
+def But(name, **kwargs):
+    kwargs["subtype"] = TestSubType.But
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
-class By(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.By
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(By, self).__init__(name, **kwargs)
+def By(name, **kwargs):
+    kwargs["subtype"] = TestSubType.By
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
-class Finally(Step):
-    def __init__(self, name, **kwargs):
-        kwargs["subtype"] = TestSubType.Finally
-        kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
-        return super(Finally, self).__init__(name, **kwargs)
+def Finally(name, **kwargs):
+    kwargs["subtype"] = TestSubType.Finally
+    kwargs["context"] = kwargs.pop("context", current().context)
+    kwargs["_frame"] = kwargs.pop("_frame", inspect.currentframe().f_back )
+    if isinstance(name, TestStep):
+        return name(**kwargs)
+    else:
+        return Step(name, **kwargs)
 
 class NullStep():
     def __enter__(self):
@@ -846,9 +981,17 @@ class _testdecorator(object):
             return parent_test
         else:
             with self.type(**_kwargs, _frame=frame) as test:
-                self.func(**{name: arg.value for name, arg in test.args.items()})
+                r = self.func(**{name: arg.value for name, arg in test.args.items()})
+                def run_generator():
+                    return next(r)
+                if inspect.isgenerator(r):
+                    run_generator()
+                    test.context.cleanup(run_generator)
             return test
 
+class TestStep(_testdecorator):
+    type = Step
+
 class TestCase(_testdecorator):
     type = Test
 
