@@ -19,15 +19,18 @@ from testflows._core.flags import Flags, SKIP
 from testflows._core.testtype import TestType, TestSubType
 from testflows._core.transform.log import message
 from testflows._core.objects import ExamplesTable
-from testflows._core.name import split, parentname
+from testflows._core.name import split, parentname, basename
 from testflows._core.cli.colors import color, cursor_up
 
 indent = " " * 2
 #: map of tests by name
-tests_by_name = {}
+tests_by_id = {}
 #: map of tests by parent
 tests_by_parent = {}
 
+def color_other(other):
+    return color(other, "white", attrs=["dim"])
+
 def color_keyword(keyword):
     return color(split(keyword)[-1], "white", attrs=["bold"])
 
@@ -55,13 +58,17 @@ def format_input(msg, keyword):
     out += color("\u270b " + msg.message, "yellow", attrs=["bold"]) + cursor_up() + "\n"
     return out
 
-def format_description(msg, indent):
-    first, rest = (msg.description.description.rstrip() + "\n").split("\n", 1)
+def format_multiline(text, indent):
+    first, rest = (text.rstrip() + "\n").split("\n", 1)
     first = first.strip()
     if first:
         first += "\n"
-    desc = f"{first}{textwrap.dedent(rest.rstrip())}".rstrip()
-    desc = textwrap.indent(desc, indent + "  ")
+    out = f"{first}{textwrap.dedent(rest.rstrip())}".rstrip()
+    out = textwrap.indent(out, indent + "  ")
+    return out
+
+def format_description(msg, indent):
+    desc = format_multiline(msg.description.description, indent)
     desc = color(desc, "white", attrs=["dim"])
     return desc + "\n"
 
@@ -110,19 +117,19 @@ def format_tickets(msg, indent):
         out.append(color(f"{indent}{' ' * 4}{ticket.name}", "white", attrs=["dim"]))
     return "\n".join(out) + "\n"
 
-def and_keyword(msg, parent_name, keyword, subtype):
+def and_keyword(msg, parent_id, keyword, subtype):
     """Handle processing of Given, When, Then, But, By and Finally
     keywords and convert them to And when necessary.
     """
-    prev = tests_by_parent[parent_name][-2] if len(tests_by_parent.get(parent_name, [])) > 1 else None
+    prev = tests_by_parent[parent_id][-2] if len(tests_by_parent.get(parent_id, [])) > 1 else None
     if prev and prev.p_subtype == subtype and tests_by_parent.get(prev.p_id) is None:
         keyword = "And"
-    parent = tests_by_name.get(parent_name)
-    if parent and parent.p_subtype == subtype and len(tests_by_parent.get(parent_name, [])) == 1:
+    parent = tests_by_id.get(parent_id)
+    if parent and parent.p_subtype == subtype and len(tests_by_parent.get(parent_id, [])) == 1:
         keyword = "And"
     return keyword
 
-def format_test(msg, keyword):
+def format_test(msg, keyword, tests_by_parent, tests_by_id):
     flags = Flags(msg.p_flags)
     if flags & SKIP and settings.show_skipped is False:
         return
@@ -132,12 +139,15 @@ def format_test(msg, keyword):
     if tests_by_parent.get(parent) is None:
         tests_by_parent[parent] = []
     tests_by_parent[parent].append(msg)
-    tests_by_name[msg.p_id] = msg
+    tests_by_id[msg.p_id] = msg
 
     if msg.p_type == TestType.Module:
         keyword += "Module"
     elif msg.p_type == TestType.Suite:
-        keyword += "Suite"
+        if msg.p_subtype == TestSubType.Feature:
+            keyword += "Feature"
+        else:
+            keyword += "Suite"
     elif msg.p_type == TestType.Iteration:
         keyword += "Iteration"
     elif msg.p_type == TestType.Step:
@@ -158,9 +168,7 @@ def format_test(msg, keyword):
         else:
             keyword += "Step"
     else:
-        if msg.p_subtype == TestSubType.Feature:
-            keyword += "Feature"
-        elif msg.p_subtype == TestSubType.Scenario:
+        if msg.p_subtype == TestSubType.Scenario:
             keyword += "Scenario"
         elif msg.p_subtype == TestSubType.Background:
             keyword += "Background"
@@ -195,11 +203,26 @@ def format_result(msg, result):
         return
 
     _result = color_result(result)
-    return f"{indent * (msg.p_id.count('/') - 1)}{_result}\n"
+    _test = color_test_name(basename(msg.test))
+
+    _indent = indent * (msg.p_id.count('/') - 1)
+    out = f"{_indent}{_result}"
+
+    if msg.name in ("Fail", "Error", "Null"):
+        out += f" {_test}"
+        if msg.message:
+            out += color_test_name(",")
+            out += f" {color(format_multiline(msg.message, _indent).lstrip(), 'yellow', attrs=['bold'])}"
+    elif msg.name.startswith("X"):
+        out += f" {_test}"
+        if msg.reason:
+            out += color_test_name(",")
+            out += f" {color(msg.reason, 'blue', attrs=['bold'])}"
+    return out + "\n"
 
 formatters = {
     message.RawInput: (format_input, f""),
-    message.RawTest: (format_test, f""),
+    message.RawTest: (format_test, f"", tests_by_parent, tests_by_id),
     message.RawResultOK: (format_result, f"OK"),
     message.RawResultFail: (format_result, f"Fail"),
     message.RawResultError: (format_result, f"Error"),
