@@ -13,11 +13,12 @@
 # limitations under the License.
 from testflows._core.test import TestBase
 from testflows._core.test import Module, Suite, Test, Step, NullStep, run
-from testflows._core.test import TestCase, TestSuite, TestModule
+from testflows._core.test import TestStep, TestCase, TestSuite, TestModule
 from testflows._core.test import Attributes, Requirements, Users, Tickets, Examples
 from testflows._core.test import Name, Description, Uid, Tags, TestClass
 from testflows._core.test import Feature, Background, Scenario
 from testflows._core.test import Given, When, Then, And, But, By, Finally
+from testflows._core.test import Steps
 from testflows._core.test import TestFeature, TestScenario
 from testflows._core.funcs import *
 from testflows._core.filters import the
